@@ -1,7 +1,7 @@
 /*
 	DDS GIMP plugin
 
-	Copyright (C) 2004 Shawn Kirst <skirst@gmail.com>,
+	Copyright (C) 2004-2012 Shawn Kirst <skirst@gmail.com>,
    with parts (C) 2003 Arne Reuter <homepage@arnereuter.de> where specified.
 
 	This program is free software; you can redistribute it and/or
@@ -16,8 +16,8 @@
 
 	You should have received a copy of the GNU General Public License
 	along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA.
+	the Free Software Foundation, 51 Franklin Street, Fifth Floor
+	Boston, MA 02110-1301, USA.
 */
 
 #include <math.h>
