@@ -1,7 +1,7 @@
 /*
 	DDS GIMP plugin
 
-	Copyright (C) 2004 Shawn Kirst <skirst@gmail.com>,
+	Copyright (C) 2004-2012 Shawn Kirst <skirst@gmail.com>,
    with parts (C) 2003 Arne Reuter <homepage@arnereuter.de> where specified.
 
 	This program is free software; you can redistribute it and/or
@@ -16,15 +16,15 @@
 
 	You should have received a copy of the GNU General Public License
 	along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA.
+	the Free Software Foundation, 51 Franklin Street, Fifth Floor
+	Boston, MA 02110-1301, USA.
 */
 
 #ifndef __DDSPLUGIN_H
 #define __DDSPLUGIN_H
 
-#define DDS_PLUGIN_VERSION_MAJOR     2
-#define DDS_PLUGIN_VERSION_MINOR     2
+#define DDS_PLUGIN_VERSION_MAJOR     3
+#define DDS_PLUGIN_VERSION_MINOR     0
 #define DDS_PLUGIN_VERSION_REVISION  1
 
 #define DDS_PLUGIN_VERSION  \
@@ -34,17 +34,20 @@
 
 typedef struct
 {
-	int compression;
-	int mipmaps;
+   int compression;
+   int mipmaps;
    int savetype;
    int format;
    int transindex;
-   int color_type;
-   int dither;
    int mipmap_filter;
+   int mipmap_wrap;
    int gamma_correct;
+   int srgb;
    float gamma;
+   int perceptual_metric;
    int show_adv_opt;
+   int preserve_alpha_coverage;
+   float alpha_test_threshold;
 } DDSWriteVals;
 
 typedef struct
