@@ -1,7 +1,7 @@
 /*
 	DDS GIMP plugin
 
-	Copyright (C) 2004 Shawn Kirst <skirst@gmail.com>,
+	Copyright (C) 2004-2012 Shawn Kirst <skirst@gmail.com>,
    with parts (C) 2003 Arne Reuter <homepage@arnereuter.de> where specified.
 
 	This program is free software; you can redistribute it and/or
@@ -16,8 +16,8 @@
 
 	You should have received a copy of the GNU General Public License
 	along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA.
+	the Free Software Foundation, 51 Franklin Street, Fifth Floor
+	Boston, MA 02110-1301, USA.
 */
 
 #include <stdio.h>
@@ -80,11 +80,14 @@ static gint is_mipmap_chain_valid = 0;
 
 static GtkWidget *compress_opt;
 static GtkWidget *format_opt;
-static GtkWidget *color_type_opt;
-static GtkWidget *dither_chk;
 static GtkWidget *mipmap_filter_opt;
+static GtkWidget *mipmap_wrap_opt;
+static GtkWidget *srgb_chk;
 static GtkWidget *gamma_chk;
 static GtkWidget *gamma_spin;
+static GtkWidget *pm_chk;
+static GtkWidget *alpha_coverage_chk;
+static GtkWidget *alpha_test_threshold_spin;
 
 typedef struct string_value_s
 {
@@ -136,23 +139,26 @@ static string_value_t mipmap_strings[] =
    {-1, 0}
 };
 
-static string_value_t color_type_strings[] =
+static string_value_t mipmap_filter_strings[] =
 {
-   {DDS_COLOR_DEFAULT,    "Default"},
-   {DDS_COLOR_DISTANCE,   "Distance"},
-   {DDS_COLOR_LUMINANCE,  "Luminance"},
-   {DDS_COLOR_INSET_BBOX, "Inset bounding box"},
+   {DDS_MIPMAP_FILTER_DEFAULT,   "Default"},
+   {DDS_MIPMAP_FILTER_NEAREST,   "Nearest"},
+   {DDS_MIPMAP_FILTER_BOX,       "Box"},
+   {DDS_MIPMAP_FILTER_TRIANGLE,  "Triangle"},
+   {DDS_MIPMAP_FILTER_QUADRATIC, "Quadratic"},
+   {DDS_MIPMAP_FILTER_BSPLINE,   "B-Spline"},
+   {DDS_MIPMAP_FILTER_MITCHELL,  "Mitchell"},
+   {DDS_MIPMAP_FILTER_LANCZOS,   "Lanczos"},
+   {DDS_MIPMAP_FILTER_KAISER,    "Kaiser"},
    {-1, 0}
 };
 
-static string_value_t mipmap_filter_strings[] =
+static string_value_t mipmap_wrap_strings[] =
 {
-   {DDS_MIPMAP_FILTER_DEFAULT,  "Default"},
-   {DDS_MIPMAP_FILTER_NEAREST,  "Nearest"},
-   {DDS_MIPMAP_FILTER_BOX,      "Box"},
-   {DDS_MIPMAP_FILTER_BILINEAR, "Bilinear"},
-   {DDS_MIPMAP_FILTER_BICUBIC,  "Bicubic"},
-   {DDS_MIPMAP_FILTER_LANCZOS,  "Lanczos"},
+   {DDS_MIPMAP_WRAP_DEFAULT, "Default"},
+   {DDS_MIPMAP_WRAP_MIRROR,  "Mirror"},
+   {DDS_MIPMAP_WRAP_REPEAT,  "Repeat"},
+   {DDS_MIPMAP_WRAP_CLAMP,   "Clamp"},
    {-1, 0}
 };
 
@@ -161,6 +167,7 @@ static string_value_t save_type_strings[] =
    {DDS_SAVE_SELECTED_LAYER, "Selected layer"},
    {DDS_SAVE_CUBEMAP,        "As cube map"},
    {DDS_SAVE_VOLUMEMAP,      "As volume map"},
+   {DDS_SAVE_ARRAY,          "As texture array"},
    {-1, 0}
 };
 
@@ -176,12 +183,12 @@ static struct
    unsigned int amask;
 } format_info[] =
 {
-   {DDS_FORMAT_RGB8,    DXGI_FORMAT_B8G8R8X8_UNORM,    3, 0, 0x00ff0000, 0x0000ff00, 0x000000ff, 0x00000000},
+   {DDS_FORMAT_RGB8,    DXGI_FORMAT_UNKNOWN,           3, 0, 0x00ff0000, 0x0000ff00, 0x000000ff, 0x00000000},
    {DDS_FORMAT_RGBA8,   DXGI_FORMAT_B8G8R8A8_UNORM,    4, 1, 0x00ff0000, 0x0000ff00, 0x000000ff, 0xff000000},
    {DDS_FORMAT_BGR8,    DXGI_FORMAT_UNKNOWN,           3, 0, 0x000000ff, 0x0000ff00, 0x00ff0000, 0x00000000},
    {DDS_FORMAT_ABGR8,   DXGI_FORMAT_R8G8B8A8_UNORM,    4, 1, 0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000},
    {DDS_FORMAT_R5G6B5,  DXGI_FORMAT_B5G6R5_UNORM,      2, 0, 0x0000f800, 0x000007e0, 0x0000001f, 0x00000000},
-   {DDS_FORMAT_RGBA4,   DXGI_FORMAT_UNKNOWN,           2, 1, 0x00000f00, 0x000000f0, 0x0000000f, 0x0000f000},
+   {DDS_FORMAT_RGBA4,   DXGI_FORMAT_B4G4R4A4_UNORM,    2, 1, 0x00000f00, 0x000000f0, 0x0000000f, 0x0000f000},
    {DDS_FORMAT_RGB5A1,  DXGI_FORMAT_B5G5R5A1_UNORM,    2, 1, 0x00007c00, 0x000003e0, 0x0000001f, 0x00008000},
    {DDS_FORMAT_RGB10A2, DXGI_FORMAT_R10G10B10A2_UNORM, 4, 1, 0x000003ff, 0x000ffc00, 0x3ff00000, 0xc0000000},
    {DDS_FORMAT_R3G3B2,  DXGI_FORMAT_UNKNOWN,           1, 0, 0x000000e0, 0x0000001c, 0x00000003, 0x00000000},
@@ -367,7 +374,7 @@ static int check_mipmap_chain_consitency(gint32 image_id)
 
    layers = gimp_image_get_layers(image_id, &num_layers);
 
-   if(num_layers == 1) return(1);
+   if(num_layers == 1) return(0);
 
    /* find largest layer */
    for(i = 0; i < num_layers; ++i)
@@ -508,15 +515,12 @@ GimpPDBStatusType write_dds(gchar *filename, gint32 image_id, gint32 drawable_id
       return(GIMP_PDB_EXECUTION_ERROR);
    }
 
-   if(interactive_dds)
-   {
-      if(strrchr(filename, '/'))
-         tmp = g_strdup_printf("Saving %s:", strrchr(filename, '/') + 1);
-      else
-         tmp = g_strdup_printf("Saving %s:", filename);
-      gimp_progress_init(tmp);
-      g_free(tmp);
-   }
+   if(strrchr(filename, '/'))
+      tmp = g_strdup_printf("Saving %s:", strrchr(filename, '/') + 1);
+   else
+      tmp = g_strdup_printf("Saving %s:", filename);
+   gimp_progress_init(tmp);
+   g_free(tmp);
 
    rc = write_image(fp, image_id, drawable_id);
 
@@ -738,6 +742,7 @@ static void write_layer(FILE *fp, gint32 image_id, gint32 drawable_id,
    unsigned char *palette = NULL;
    int i, c, x, y, size, fmtsize, offset, colors;
    int compression = dds_write_vals.compression;
+   int flags = 0;
 
    basetype = gimp_image_base_type(image_id);
    type = gimp_drawable_type(drawable_id);
@@ -868,8 +873,11 @@ static void write_layer(FILE *fp, gint32 image_id, gint32 drawable_id,
             generate_mipmaps(dst, src, w, h, bpp, palette != NULL,
                              mipmaps,
                              dds_write_vals.mipmap_filter,
-                             dds_write_vals.gamma_correct,
-                             dds_write_vals.gamma);
+                             dds_write_vals.mipmap_wrap,
+                             dds_write_vals.gamma_correct + dds_write_vals.srgb,
+                             dds_write_vals.gamma,
+                             dds_write_vals.preserve_alpha_coverage,
+                             dds_write_vals.alpha_test_threshold);
          }
          else
          {
@@ -944,7 +952,11 @@ static void write_layer(FILE *fp, gint32 image_id, gint32 drawable_id,
          {
             generate_mipmaps(fmtdst, src, w, h, bpp, 0, mipmaps,
                              dds_write_vals.mipmap_filter,
-                             dds_write_vals.gamma_correct, dds_write_vals.gamma);
+                             dds_write_vals.mipmap_wrap,
+                             dds_write_vals.gamma_correct + dds_write_vals.srgb,
+                             dds_write_vals.gamma,
+                             dds_write_vals.preserve_alpha_coverage,
+                             dds_write_vals.alpha_test_threshold);
          }
          else
          {
@@ -956,8 +968,10 @@ static void write_layer(FILE *fp, gint32 image_id, gint32 drawable_id,
          src = fmtdst;
       }
 
-      dxt_compress(dst, src, compression, w, h, bpp, mipmaps,
-                   dds_write_vals.color_type, dds_write_vals.dither);
+      flags = 0;
+      if(dds_write_vals.perceptual_metric) flags |= DXT_PERCEPTUAL;
+
+      dxt_compress(dst, src, compression, w, h, bpp, mipmaps, flags);
 
       fwrite(dst, 1, size, fp);
 
@@ -1039,7 +1053,8 @@ static void write_volume_mipmaps(FILE *fp, gint32 image_id, gint32 *layers,
    generate_volume_mipmaps(dst, src, w, h, d, bpp,
                            palette != NULL, mipmaps,
                            dds_write_vals.mipmap_filter,
-                           dds_write_vals.gamma_correct,
+                           dds_write_vals.mipmap_wrap,
+                           dds_write_vals.gamma_correct + dds_write_vals.srgb,
                            dds_write_vals.gamma);
 
    if(dds_write_vals.format > DDS_FORMAT_DEFAULT)
@@ -1070,14 +1085,16 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
    GimpPixelRgn rgn;
    int i, w, h, bpp = 0, fmtbpp = 0, has_alpha = 0;
    int num_mipmaps;
-   unsigned char hdr[DDS_HEADERSIZE];
+   unsigned char hdr[DDS_HEADERSIZE], hdr10[DDS_HEADERSIZE_DX10];
    unsigned int flags = 0, pflags = 0, caps = 0, caps2 = 0, size = 0;
    unsigned int rmask = 0, gmask = 0, bmask = 0, amask = 0;
    unsigned int fourcc = 0;
+   DXGI_FORMAT dxgi_format = DXGI_FORMAT_UNKNOWN;
    gint32 num_layers, *layers;
    guchar *cmap;
    gint colors;
    unsigned char zero[4] = {0, 0, 0, 0};
+   int is_dx10 = 0, array_size = 1;
 
    layers = gimp_image_get_layers(image_id, &num_layers);
 
@@ -1114,6 +1131,7 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
             gmask = format_info[i].gmask;
             bmask = format_info[i].bmask;
             amask = format_info[i].amask;
+            dxgi_format = format_info[i].dxgi_format;
             break;
          }
       }
@@ -1132,6 +1150,7 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
          has_alpha = 0;
          rmask = 0x000000ff;
          gmask = bmask = amask = 0;
+         dxgi_format = DXGI_FORMAT_R8_UNORM;
       }
    }
    else if(bpp == 2)
@@ -1168,6 +1187,7 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
       gmask = 0x0000ff00;
       bmask = 0x000000ff;
       amask = 0xff000000;
+      dxgi_format = DXGI_FORMAT_B8G8R8A8_UNORM;
    }
 
    memset(hdr, 0, DDS_HEADERSIZE);
@@ -1177,19 +1197,23 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
    PUTL32(hdr + 12,  h);
    PUTL32(hdr + 16,  w);
    PUTL32(hdr + 76,  32);
-   PUTL32(hdr + 88,  fmtbpp << 3);
-   PUTL32(hdr + 92,  rmask);
-   PUTL32(hdr + 96,  gmask);
-   PUTL32(hdr + 100, bmask);
-   PUTL32(hdr + 104, amask);
+
+   if(dds_write_vals.compression == DDS_COMPRESS_NONE)
+   {
+      PUTL32(hdr + 88,  fmtbpp << 3);
+      PUTL32(hdr + 92,  rmask);
+      PUTL32(hdr + 96,  gmask);
+      PUTL32(hdr + 100, bmask);
+      PUTL32(hdr + 104, amask);
+   }
 
    /*
     put some information in the reserved area to identify the origin
     of the image
    */
-   PUTL32(hdr + 32,  FOURCC('G','I','M','P'));
-   PUTL32(hdr + 36,  FOURCC('-','D','D','S'));
-   PUTL32(hdr + 40,  DDS_PLUGIN_VERSION);
+   PUTL32(hdr + 32, FOURCC('G','I','M','P'));
+   PUTL32(hdr + 36, FOURCC('-','D','D','S'));
+   PUTL32(hdr + 40, DDS_PLUGIN_VERSION);
 
    flags = DDSD_CAPS | DDSD_PIXELFORMAT | DDSD_WIDTH | DDSD_HEIGHT;
 
@@ -1206,14 +1230,14 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
    else
       num_mipmaps = 1;
 
-   if(dds_write_vals.savetype == DDS_SAVE_CUBEMAP && is_cubemap)
+   if((dds_write_vals.savetype == DDS_SAVE_CUBEMAP) && is_cubemap)
    {
       caps |= DDSCAPS_COMPLEX;
       caps2 |= (DDSCAPS2_CUBEMAP | DDSCAPS2_CUBEMAP_ALL_FACES);
    }
-   else if(dds_write_vals.savetype == DDS_SAVE_VOLUMEMAP && is_volume)
+   else if((dds_write_vals.savetype == DDS_SAVE_VOLUMEMAP) && is_volume)
    {
-      PUTL32(hdr + 24, num_layers);
+      PUTL32(hdr + 24, num_layers); /* depth */
       flags |= DDSD_DEPTH;
       caps |= DDSCAPS_COMPLEX;
       caps2 |= DDSCAPS2_VOLUME;
@@ -1233,7 +1257,7 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
             pflags |= DDPF_ALPHA;
          else
          {
-            if((fmtbpp == 1 || dds_write_vals.format == DDS_FORMAT_L8A8) &&
+            if(((fmtbpp == 1) || (dds_write_vals.format == DDS_FORMAT_L8A8)) &&
                (dds_write_vals.format != DDS_FORMAT_R3G3B2))
                pflags |= DDPF_LUMINANCE;
             else
@@ -1249,7 +1273,7 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
             else
                pflags |= DDPF_LUMINANCE;
          }
-         else if(bpp == 2 && basetype == GIMP_INDEXED)
+         else if((bpp == 2) && (basetype == GIMP_INDEXED))
             pflags |= DDPF_PALETTEINDEXED8;
          else
             pflags |= DDPF_RGB;
@@ -1257,8 +1281,8 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
 
       if(has_alpha) pflags |= DDPF_ALPHAPIXELS;
 
-      PUTL32(hdr + 8, flags);
-      PUTL32(hdr + 20, w * fmtbpp);
+      PUTL32(hdr + 8,  flags);
+      PUTL32(hdr + 20, w * fmtbpp); /* pitch */
       PUTL32(hdr + 80, pflags);
 
       /*
@@ -1283,21 +1307,35 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
       switch(dds_write_vals.compression)
       {
          case DDS_COMPRESS_BC1:
-            fourcc = FOURCC('D','X','T','1'); break;
+            fourcc = FOURCC('D','X','T','1');
+            dxgi_format = DXGI_FORMAT_BC1_UNORM;
+            break;
          case DDS_COMPRESS_BC2:
-            fourcc = FOURCC('D','X','T','3'); break;
+            fourcc = FOURCC('D','X','T','3');
+            dxgi_format = DXGI_FORMAT_BC2_UNORM;
+            break;
          case DDS_COMPRESS_BC3:
          case DDS_COMPRESS_BC3N:
          case DDS_COMPRESS_YCOCG:
          case DDS_COMPRESS_YCOCGS:
          case DDS_COMPRESS_AEXP:
-            fourcc = FOURCC('D','X','T','5'); break;
+            fourcc = FOURCC('D','X','T','5');
+            dxgi_format = DXGI_FORMAT_BC3_UNORM;
+            break;
          case DDS_COMPRESS_RXGB:
-            fourcc = FOURCC('R','X','G','B'); break;
+            fourcc = FOURCC('R','X','G','B');
+            dxgi_format = DXGI_FORMAT_BC3_UNORM;
+            break;
          case DDS_COMPRESS_BC4:
-            fourcc = FOURCC('A','T','I','1'); break;
+            fourcc = FOURCC('A','T','I','1');
+            dxgi_format = DXGI_FORMAT_BC4_UNORM;
+            //is_dx10 = 1;
+            break;
          case DDS_COMPRESS_BC5:
-            fourcc = FOURCC('A','T','I','2'); break;
+            fourcc = FOURCC('A','T','I','2');
+            dxgi_format = DXGI_FORMAT_BC5_UNORM;
+            //is_dx10 = 1;
+            break;
       }
 
       if((dds_write_vals.compression == DDS_COMPRESS_BC3N) ||
@@ -1309,13 +1347,13 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
       PUTL32(hdr + 84, fourcc);
 
       size = ((w + 3) >> 2) * ((h + 3) >> 2);
-      if(dds_write_vals.compression == DDS_COMPRESS_BC1 ||
-         dds_write_vals.compression == DDS_COMPRESS_BC4)
+      if((dds_write_vals.compression == DDS_COMPRESS_BC1) ||
+         (dds_write_vals.compression == DDS_COMPRESS_BC4))
          size *= 8;
       else
          size *= 16;
 
-      PUTL32(hdr + 20, size);
+      PUTL32(hdr + 20, size); /* linear size */
 
       /*
        write extra fourcc info - this is special to GIMP DDS. When the image
@@ -1336,10 +1374,34 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
       }
    }
 
+   /* texture arrays require a DX10 header */
+   if(dds_write_vals.savetype == DDS_SAVE_ARRAY)
+      is_dx10 = 1;
+
+   if(is_dx10)
+   {
+      array_size = (dds_write_vals.savetype == DDS_SAVE_SELECTED_LAYER) ? 1 : num_layers;
+
+      PUTL32(hdr10 +  0, dxgi_format);
+      PUTL32(hdr10 +  4, D3D10_RESOURCE_DIMENSION_TEXTURE2D);
+      PUTL32(hdr10 +  8, 0);
+      PUTL32(hdr10 + 12, array_size);
+      PUTL32(hdr10 + 16, 0);
+
+      /* update main header accordingly */
+      PUTL32(hdr + 80, pflags | DDPF_FOURCC);
+      PUTL32(hdr + 84, FOURCC('D','X','1','0'));
+   }
+
    fwrite(hdr, DDS_HEADERSIZE, 1, fp);
 
-   if(basetype == GIMP_INDEXED && dds_write_vals.format == DDS_FORMAT_DEFAULT &&
-      dds_write_vals.compression == DDS_COMPRESS_NONE)
+   if(is_dx10)
+      fwrite(hdr10, DDS_HEADERSIZE_DX10, 1, fp);
+
+   /* write palette for indexed images */
+   if((basetype == GIMP_INDEXED) &&
+      (dds_write_vals.format == DDS_FORMAT_DEFAULT) &&
+      (dds_write_vals.compression == DDS_COMPRESS_NONE))
    {
       cmap = gimp_image_get_colormap(image_id, &colors);
       for(i = 0; i < colors; ++i)
@@ -1360,8 +1422,7 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
       {
          write_layer(fp, image_id, cubemap_faces[i], w, h, bpp, fmtbpp,
                      num_mipmaps);
-         if(interactive_dds)
-            gimp_progress_update((float)(i + 1) / 6.0);
+         gimp_progress_update((float)(i + 1) / 6.0);
       }
    }
    else if(dds_write_vals.savetype == DDS_SAVE_VOLUMEMAP)
@@ -1369,22 +1430,27 @@ static int write_image(FILE *fp, gint32 image_id, gint32 drawable_id)
       for(i = 0; i < num_layers; ++i)
       {
          write_layer(fp, image_id, layers[i], w, h, bpp, fmtbpp, 1);
-         if(interactive_dds)
-            gimp_progress_update((float)i / (float)num_layers);
+         gimp_progress_update((float)i / (float)num_layers);
       }
 
       if(num_mipmaps > 1)
          write_volume_mipmaps(fp, image_id, layers, w, h, num_layers,
                               bpp, fmtbpp, num_mipmaps);
    }
+   else if(dds_write_vals.savetype == DDS_SAVE_ARRAY)
+   {
+      for(i = 0; i < num_layers; ++i)
+      {
+         write_layer(fp, image_id, layers[i], w, h, bpp, fmtbpp, num_mipmaps);
+         gimp_progress_update((float)i / (float)num_layers);
+      }
+   }
    else
    {
-      write_layer(fp, image_id, drawable_id, w, h, bpp, fmtbpp,
-                  num_mipmaps);
+      write_layer(fp, image_id, drawable_id, w, h, bpp, fmtbpp, num_mipmaps);
    }
 
-   if(interactive_dds)
-      gimp_progress_update(1.0);
+   gimp_progress_update(1.0);
 
    gimp_drawable_detach(drawable);
 
@@ -1505,16 +1571,7 @@ static void compression_selected(GtkWidget *widget, gpointer data)
                       &dds_write_vals.compression, -1);
 
    gtk_widget_set_sensitive(format_opt, dds_write_vals.compression == DDS_COMPRESS_NONE);
-   gtk_widget_set_sensitive(color_type_opt,
-                            (dds_write_vals.compression != DDS_COMPRESS_NONE) &&
-                            (dds_write_vals.compression != DDS_COMPRESS_BC4) &&
-                            (dds_write_vals.compression != DDS_COMPRESS_BC5) &&
-                            (dds_write_vals.compression != DDS_COMPRESS_YCOCGS));
-   gtk_widget_set_sensitive(dither_chk,
-                            (dds_write_vals.compression != DDS_COMPRESS_NONE) &&
-                            (dds_write_vals.compression != DDS_COMPRESS_BC4) &&
-                            (dds_write_vals.compression != DDS_COMPRESS_BC5) &&
-                            (dds_write_vals.compression != DDS_COMPRESS_YCOCGS));
+   gtk_widget_set_sensitive(pm_chk, dds_write_vals.compression != DDS_COMPRESS_NONE);
 }
 
 static void savetype_selected(GtkWidget *widget, gpointer data)
@@ -1525,23 +1582,14 @@ static void savetype_selected(GtkWidget *widget, gpointer data)
    {
       case DDS_SAVE_SELECTED_LAYER:
       case DDS_SAVE_CUBEMAP:
+      case DDS_SAVE_ARRAY:
          gtk_widget_set_sensitive(compress_opt, 1);
-         string_value_combo_set_item_sensitive(mipmap_filter_opt,
-                                               DDS_MIPMAP_FILTER_LANCZOS,
-                                               1);
          break;
       case DDS_SAVE_VOLUMEMAP:
          dds_write_vals.compression = DDS_COMPRESS_NONE;
          gtk_combo_box_set_active(GTK_COMBO_BOX(compress_opt),
                                   DDS_COMPRESS_NONE);
          gtk_widget_set_sensitive(compress_opt, 0);
-         if(dds_write_vals.mipmap_filter == DDS_MIPMAP_FILTER_LANCZOS)
-            dds_write_vals.mipmap_filter = DDS_MIPMAP_FILTER_DEFAULT;
-         string_value_combo_set_active(mipmap_filter_opt,
-                                       dds_write_vals.mipmap_filter);
-         string_value_combo_set_item_sensitive(mipmap_filter_opt,
-                                               DDS_MIPMAP_FILTER_LANCZOS,
-                                               0);
          break;
    }
 }
@@ -1556,8 +1604,14 @@ static void mipmaps_selected(GtkWidget *widget, gpointer data)
                       &dds_write_vals.mipmaps, -1);
 
    gtk_widget_set_sensitive(mipmap_filter_opt, dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE);
+   gtk_widget_set_sensitive(mipmap_wrap_opt, dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE);
    gtk_widget_set_sensitive(gamma_chk, dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE);
-   gtk_widget_set_sensitive(gamma_spin, (dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE) && dds_write_vals.gamma_correct);
+   gtk_widget_set_sensitive(srgb_chk, (dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE) && dds_write_vals.gamma_correct);
+   gtk_widget_set_sensitive(gamma_spin, (dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE) &&
+                            dds_write_vals.gamma_correct && !dds_write_vals.srgb);
+   gtk_widget_set_sensitive(alpha_coverage_chk, dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE);
+   gtk_widget_set_sensitive(alpha_test_threshold_spin, (dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE) &&
+                            dds_write_vals.preserve_alpha_coverage);
 }
 
 static void toggle_clicked(GtkWidget *widget, gpointer data)
@@ -1596,7 +1650,14 @@ static void adv_opt_expanded(GtkWidget *widget, gpointer data)
 static void gamma_correct_clicked(GtkWidget *widget, gpointer data)
 {
    dds_write_vals.gamma_correct = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
-   gtk_widget_set_sensitive(gamma_spin, dds_write_vals.gamma_correct);
+   gtk_widget_set_sensitive(srgb_chk, dds_write_vals.gamma_correct);
+   gtk_widget_set_sensitive(gamma_spin, dds_write_vals.gamma_correct && !dds_write_vals.srgb);
+}
+
+static void srgb_clicked(GtkWidget *widget, gpointer data)
+{
+   dds_write_vals.srgb = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+   gtk_widget_set_sensitive(gamma_spin, !dds_write_vals.srgb);
 }
 
 static void gamma_changed(GtkWidget *widget, gpointer data)
@@ -1604,23 +1665,31 @@ static void gamma_changed(GtkWidget *widget, gpointer data)
    dds_write_vals.gamma = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
 }
 
+static void alpha_coverage_clicked(GtkWidget *widget, gpointer data)
+{
+   dds_write_vals.preserve_alpha_coverage = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+   gtk_widget_set_sensitive(alpha_test_threshold_spin, dds_write_vals.preserve_alpha_coverage);
+}
+
+static void alpha_test_threshold_changed(GtkWidget *widget, gpointer data)
+{
+   dds_write_vals.alpha_test_threshold = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
+}
+
 static gint save_dialog(gint32 image_id, gint32 drawable_id)
 {
    GtkWidget *dlg;
-   GtkWidget *vbox, *hbox;
+   GtkWidget *vbox, *vbox2, *hbox;
    GtkWidget *table;
    GtkWidget *label;
    GtkWidget *opt;
    GtkWidget *check;
    GtkWidget *spin;
    GtkWidget *expander;
+   GtkWidget *frame;
    GimpImageBaseType basetype;
 
-   if(is_cubemap)
-      dds_write_vals.savetype = DDS_SAVE_CUBEMAP;
-   else if(is_volume)
-      dds_write_vals.savetype = DDS_SAVE_VOLUMEMAP;
-   else
+   if(is_cubemap || is_volume)
       dds_write_vals.savetype = DDS_SAVE_SELECTED_LAYER;
 
    basetype = gimp_image_base_type(image_id);
@@ -1640,7 +1709,7 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
 
    gtk_window_set_resizable(GTK_WINDOW(dlg), 0);
 
-   vbox = gtk_vbox_new(0, 8);
+   vbox = gtk_vbox_new(0, 4);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 8);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), vbox, 1, 1, 0);
    gtk_widget_show(vbox);
@@ -1648,7 +1717,7 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
    table = gtk_table_new(4, 2, 0);
    gtk_widget_show(table);
    gtk_box_pack_start(GTK_BOX(vbox), table, 1, 1, 0);
-   gtk_table_set_row_spacings(GTK_TABLE(table), 8);
+   gtk_table_set_row_spacings(GTK_TABLE(table), 4);
    gtk_table_set_col_spacings(GTK_TABLE(table), 8);
 
    label = gtk_label_new("Compression:");
@@ -1656,7 +1725,7 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                     (GtkAttachOptions)(GTK_FILL),
                     (GtkAttachOptions)(0), 0, 0);
-   gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+   gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
    opt = string_value_combo_new(compression_strings,
                                 dds_write_vals.compression);
@@ -1675,7 +1744,7 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
                     (GtkAttachOptions)(GTK_FILL),
                     (GtkAttachOptions)(0), 0, 0);
-   gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+   gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
    opt = string_value_combo_new(format_strings, dds_write_vals.format);
    gtk_widget_show(opt);
@@ -1696,7 +1765,7 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
                     (GtkAttachOptions)(GTK_FILL),
                     (GtkAttachOptions)(0), 0, 0);
-   gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+   gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
    opt = string_value_combo_new(save_type_strings, dds_write_vals.savetype);
    gtk_widget_show(opt);
@@ -1709,6 +1778,7 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
 
    string_value_combo_set_item_sensitive(opt, DDS_SAVE_CUBEMAP, is_cubemap);
    string_value_combo_set_item_sensitive(opt, DDS_SAVE_VOLUMEMAP, is_volume);
+   string_value_combo_set_item_sensitive(opt, DDS_SAVE_ARRAY, is_volume);
 
    gtk_widget_set_sensitive(opt, is_cubemap || is_volume);
 
@@ -1717,7 +1787,7 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,
                     (GtkAttachOptions)(GTK_FILL),
                     (GtkAttachOptions)(0), 0, 0);
-   gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+   gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
    opt = string_value_combo_new(mipmap_strings,
                                 dds_write_vals.mipmaps);
@@ -1785,68 +1855,87 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
    gtk_box_pack_start(GTK_BOX(vbox), expander, 1, 1, 0);
    gtk_widget_show(expander);
 
-   table = gtk_table_new(5, 2, 0);
-   gtk_table_set_row_spacings(GTK_TABLE(table), 8);
+
+   vbox2 = gtk_vbox_new(0, 4);
+   gtk_container_add(GTK_CONTAINER(expander), vbox2);
+   gtk_widget_show(vbox2);
+
+   frame = gtk_frame_new("Compression");
+   gtk_box_pack_start(GTK_BOX(vbox2), frame, 1, 1, 0);
+   gtk_widget_show(frame);
+
+   table = gtk_table_new(1, 2, 0);
+   gtk_table_set_row_spacings(GTK_TABLE(table), 4);
    gtk_table_set_col_spacings(GTK_TABLE(table), 8);
-   gtk_container_add(GTK_CONTAINER(expander), table);
+   gtk_container_set_border_width(GTK_CONTAINER(table), 8);
+   gtk_container_add(GTK_CONTAINER(frame), table);
    gtk_widget_show(table);
 
-   label = gtk_label_new("Color selection:");
+   check = gtk_check_button_new_with_label("Use perceptual error metric");
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), dds_write_vals.perceptual_metric);
+   gtk_table_attach(GTK_TABLE(table), check, 0, 2, 0, 1,
+                    (GtkAttachOptions)(GTK_FILL),
+                    (GtkAttachOptions)(0), 0, 0);
+   gtk_signal_connect(GTK_OBJECT(check), "clicked",
+                      GTK_SIGNAL_FUNC(toggle_clicked), &dds_write_vals.perceptual_metric);
+   gtk_widget_show(check);
+
+   pm_chk = check;
+
+   frame = gtk_frame_new("Mipmaps");
+   gtk_box_pack_start(GTK_BOX(vbox2), frame, 1, 1, 0);
+   gtk_widget_show(frame);
+
+   table = gtk_table_new(7, 2, 0);
+   gtk_table_set_row_spacings(GTK_TABLE(table), 4);
+   gtk_table_set_col_spacings(GTK_TABLE(table), 8);
+   gtk_container_set_border_width(GTK_CONTAINER(table), 8);
+   gtk_container_add(GTK_CONTAINER(frame), table);
+   gtk_widget_show(table);
+
+   label = gtk_label_new("Filter:");
    gtk_widget_show(label);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                     (GtkAttachOptions)(GTK_FILL),
                     (GtkAttachOptions)(0), 0, 0);
-   gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+   gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
-   opt = string_value_combo_new(color_type_strings, dds_write_vals.color_type);
+   opt = string_value_combo_new(mipmap_filter_strings,
+                                dds_write_vals.mipmap_filter);
    gtk_widget_show(opt);
    gtk_table_attach(GTK_TABLE(table), opt, 1, 2, 0, 1,
                     (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
                     (GtkAttachOptions)(GTK_EXPAND), 0, 0);
 
-   gtk_signal_connect(GTK_OBJECT(opt), "changed",
-                      GTK_SIGNAL_FUNC(string_value_combo_selected),
-                      &dds_write_vals.color_type);
-
-   color_type_opt = opt;
-
-   check = gtk_check_button_new_with_label("Use dithering");
-   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), dds_write_vals.dither);
-   gtk_table_attach(GTK_TABLE(table), check, 0, 2, 1, 2,
-                    (GtkAttachOptions)(GTK_FILL),
-                    (GtkAttachOptions)(0), 0, 0);
-   gtk_signal_connect(GTK_OBJECT(check), "clicked",
-                      GTK_SIGNAL_FUNC(toggle_clicked), &dds_write_vals.dither);
-   gtk_widget_show(check);
-
-   dither_chk = check;
-
-   label = gtk_label_new("Mipmap filter:");
-   gtk_widget_show(label);
-   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
-                    (GtkAttachOptions)(GTK_FILL),
-                    (GtkAttachOptions)(0), 0, 0);
-   gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-
-   opt = string_value_combo_new(mipmap_filter_strings,
-                                dds_write_vals.mipmap_filter);
-   gtk_widget_show(opt);
-   gtk_table_attach(GTK_TABLE(table), opt, 1, 2, 2, 3,
-                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
-                    (GtkAttachOptions)(GTK_EXPAND), 0, 0);
-
-   if(is_volume && dds_write_vals.savetype == DDS_SAVE_VOLUMEMAP)
-      string_value_combo_set_item_sensitive(opt, DDS_MIPMAP_FILTER_LANCZOS, 0);
-
    gtk_signal_connect(GTK_OBJECT(opt), "changed",
                       GTK_SIGNAL_FUNC(string_value_combo_selected),
                       &dds_write_vals.mipmap_filter);
 
    mipmap_filter_opt = opt;
 
-   check = gtk_check_button_new_with_label("Gamma-correct mipmaps");
+   label = gtk_label_new("Wrap mode:");
+   gtk_widget_show(label);
+   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
+                    (GtkAttachOptions)(GTK_FILL),
+                    (GtkAttachOptions)(0), 0, 0);
+   gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+
+   opt = string_value_combo_new(mipmap_wrap_strings,
+                                dds_write_vals.mipmap_wrap);
+   gtk_widget_show(opt);
+   gtk_table_attach(GTK_TABLE(table), opt, 1, 2, 1, 2,
+                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions)(GTK_EXPAND), 0, 0);
+
+   gtk_signal_connect(GTK_OBJECT(opt), "changed",
+                      GTK_SIGNAL_FUNC(string_value_combo_selected),
+                      &dds_write_vals.mipmap_wrap);
+
+   mipmap_wrap_opt = opt;
+
+   check = gtk_check_button_new_with_label("Apply gamma correction");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), dds_write_vals.gamma_correct && dds_write_vals.mipmaps);
-   gtk_table_attach(GTK_TABLE(table), check, 0, 2, 3, 4,
+   gtk_table_attach(GTK_TABLE(table), check, 1, 2, 2, 3,
                     (GtkAttachOptions)(GTK_FILL),
                     (GtkAttachOptions)(0), 0, 0);
    gtk_signal_connect(GTK_OBJECT(check), "clicked",
@@ -1855,12 +1944,23 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
 
    gamma_chk = check;
 
+   check = gtk_check_button_new_with_label("Use sRGB colorspace");
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), dds_write_vals.gamma_correct && dds_write_vals.srgb);
+   gtk_table_attach(GTK_TABLE(table), check, 1, 2, 3, 4,
+                    (GtkAttachOptions)(GTK_FILL),
+                    (GtkAttachOptions)(0), 0, 0);
+   gtk_signal_connect(GTK_OBJECT(check), "clicked",
+                      GTK_SIGNAL_FUNC(srgb_clicked), NULL);
+   gtk_widget_show(check);
+
+   srgb_chk = check;
+
    label = gtk_label_new("Gamma:");
    gtk_widget_show(label);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5,
                     (GtkAttachOptions)(GTK_FILL),
                     (GtkAttachOptions)(0), 0, 0);
-   gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+   gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
 
    spin = gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(dds_write_vals.gamma, 1e-05, 100, 0.1, 0.5, 0)), 1, 1);
    gtk_table_attach(GTK_TABLE(table), spin, 1, 2, 4, 5,
@@ -1873,19 +1973,45 @@ static gint save_dialog(gint32 image_id, gint32 drawable_id)
 
    gamma_spin = spin;
 
-   gtk_widget_set_sensitive(color_type_opt,
-                            (dds_write_vals.compression != DDS_COMPRESS_NONE) &&
-                            (dds_write_vals.compression != DDS_COMPRESS_BC4) &&
-                            (dds_write_vals.compression != DDS_COMPRESS_BC5) &&
-                            (dds_write_vals.compression != DDS_COMPRESS_YCOCGS));
-   gtk_widget_set_sensitive(dither_chk,
-                            (dds_write_vals.compression != DDS_COMPRESS_NONE) &&
-                            (dds_write_vals.compression != DDS_COMPRESS_BC4) &&
-                            (dds_write_vals.compression != DDS_COMPRESS_BC5) &&
-                            (dds_write_vals.compression != DDS_COMPRESS_YCOCGS));
+   check = gtk_check_button_new_with_label("Preserve alpha test coverage");
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), dds_write_vals.preserve_alpha_coverage && dds_write_vals.mipmaps);
+   gtk_table_attach(GTK_TABLE(table), check, 1, 2, 5, 6,
+                    (GtkAttachOptions)(GTK_FILL),
+                    (GtkAttachOptions)(0), 0, 0);
+   gtk_signal_connect(GTK_OBJECT(check), "clicked",
+                      GTK_SIGNAL_FUNC(alpha_coverage_clicked), NULL);
+   gtk_widget_show(check);
+
+   alpha_coverage_chk = check;
+
+   label = gtk_label_new("Alpha test threshold:");
+   gtk_widget_show(label);
+   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 6, 7,
+                    (GtkAttachOptions)(GTK_FILL),
+                    (GtkAttachOptions)(0), 0, 0);
+   gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+
+   spin = gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(dds_write_vals.alpha_test_threshold, 0, 1, 0.01, 0.1, 0)), 1, 2);
+   gtk_table_attach(GTK_TABLE(table), spin, 1, 2, 6, 7,
+                    (GtkAttachOptions)(GTK_EXPAND | GTK_FILL),
+                    (GtkAttachOptions)(GTK_EXPAND), 0, 0);
+   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(spin), GTK_UPDATE_IF_VALID);
+   gtk_signal_connect(GTK_OBJECT(spin), "value_changed",
+                      GTK_SIGNAL_FUNC(alpha_test_threshold_changed), 0);
+   gtk_widget_show(spin);
+
+   alpha_test_threshold_spin = spin;
+
    gtk_widget_set_sensitive(mipmap_filter_opt, dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE);
+   gtk_widget_set_sensitive(mipmap_wrap_opt, dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE);
    gtk_widget_set_sensitive(gamma_chk, dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE);
-   gtk_widget_set_sensitive(gamma_spin, (dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE) && dds_write_vals.gamma_correct);
+   gtk_widget_set_sensitive(srgb_chk, (dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE) && dds_write_vals.gamma_correct);
+   gtk_widget_set_sensitive(gamma_spin, (dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE) &&
+                            dds_write_vals.gamma_correct && !dds_write_vals.srgb);
+   gtk_widget_set_sensitive(pm_chk, dds_write_vals.compression != DDS_COMPRESS_NONE);
+   gtk_widget_set_sensitive(alpha_coverage_chk, dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE);
+   gtk_widget_set_sensitive(alpha_test_threshold_spin, (dds_write_vals.mipmaps == DDS_MIPMAP_GENERATE) &&
+                            dds_write_vals.preserve_alpha_coverage);
 
    gtk_widget_show(dlg);
 
